###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/es-ES/2021.json
# https://holidata.net/es-ES/2022.json
#
# Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.es-ES1.name=Año Nuevo
holiday.es-ES1.date=20210101
holiday.es-ES2.name=Epifanía del Señor
holiday.es-ES2.date=20210106
holiday.es-ES3.name=Día de Andalucía
holiday.es-ES3.date=20210301
holiday.es-ES4.name=Día de las Illes Balears
holiday.es-ES4.date=20210301
holiday.es-ES5.name=Estatuto de Autonomía de la Ciudad de Melilla
holiday.es-ES5.date=20210313
holiday.es-ES6.name=San José
holiday.es-ES6.date=20210319
holiday.es-ES7.name=Jueves Santo
holiday.es-ES7.date=20210401
holiday.es-ES8.name=Viernes Santo
holiday.es-ES8.date=20210402
holiday.es-ES9.name=Pascua
holiday.es-ES9.date=20210404
holiday.es-ES10.name=Lunes de Pascua
holiday.es-ES10.date=20210405
holiday.es-ES11.name=Fiesta de Castilla y León
holiday.es-ES11.date=20210423
holiday.es-ES12.name=San Jorge / Día de Aragón
holiday.es-ES12.date=20210423
holiday.es-ES13.name=Fiesta del Trabajo
holiday.es-ES13.date=20210501
holiday.es-ES14.name=Lunes siguiente al Día de la Comunidad de Madrid
holiday.es-ES14.date=20210503
holiday.es-ES15.name=Día de las Letras Gallegas
holiday.es-ES15.date=20210517
holiday.es-ES16.name=Día de Castilla-La Mancha
holiday.es-ES16.date=20210531
holiday.es-ES17.name=Corpus Christi
holiday.es-ES17.date=20210603
holiday.es-ES18.name=Día de la Región de Murcia
holiday.es-ES18.date=20210609
holiday.es-ES19.name=Día de La Rioja
holiday.es-ES19.date=20210609
holiday.es-ES20.name=San Juan
holiday.es-ES20.date=20210624
holiday.es-ES21.name=Fiesta del Sacrificio (Eidul Adha)
holiday.es-ES21.date=20210720
holiday.es-ES22.name=Fiesta del Sacrificio (Aid El Kebir)
holiday.es-ES22.date=20210721
holiday.es-ES23.name=Santiago Apóstol / Día Nacional de Galicia
holiday.es-ES23.date=20210725
holiday.es-ES24.name=Día de las Instituciones de Cantabria
holiday.es-ES24.date=20210728
holiday.es-ES25.name=Asunción de la Virgen
holiday.es-ES25.date=20210815
holiday.es-ES26.name=Lunes siguiente a la Asunción de la Virgen
holiday.es-ES26.date=20210816
holiday.es-ES27.name=Día de Ceuta
holiday.es-ES27.date=20210902
holiday.es-ES28.name=Día de Asturias
holiday.es-ES28.date=20210908
holiday.es-ES29.name=Día de Extremadura
holiday.es-ES29.date=20210908
holiday.es-ES30.name=Fiesta Nacional de Cataluña
holiday.es-ES30.date=20210911
holiday.es-ES31.name=La Bien Aparecida
holiday.es-ES31.date=20210915
holiday.es-ES32.name=Día de la Comunitat Valenciana
holiday.es-ES32.date=20211009
holiday.es-ES33.name=Fiesta Nacional de España
holiday.es-ES33.date=20211012
holiday.es-ES34.name=Todos los Santos
holiday.es-ES34.date=20211101
holiday.es-ES35.name=Día de la Constitución Española
holiday.es-ES35.date=20211206
holiday.es-ES36.name=Inmaculada Concepción
holiday.es-ES36.date=20211208
holiday.es-ES37.name=Natividad del Señor
holiday.es-ES37.date=20211225
holiday.es-ES38.name=Año Nuevo
holiday.es-ES38.date=20220101
holiday.es-ES39.name=Epifanía del Señor
holiday.es-ES39.date=20220106
holiday.es-ES40.name=Día de Andalucía
holiday.es-ES40.date=20220228
holiday.es-ES41.name=Estatuto de Autonomía de la Ciudad de Melilla
holiday.es-ES41.date=20220313
holiday.es-ES42.name=Viernes Santo
holiday.es-ES42.date=20220415
holiday.es-ES43.name=Pascua
holiday.es-ES43.date=20220417
holiday.es-ES44.name=Fiesta de Castilla y León
holiday.es-ES44.date=20220423
holiday.es-ES45.name=San Jorge / Día de Aragón
holiday.es-ES45.date=20220423
holiday.es-ES46.name=Fiesta del Trabajo
holiday.es-ES46.date=20220501
holiday.es-ES47.name=Fiesta de la Comunidad de Madrid
holiday.es-ES47.date=20220502
holiday.es-ES48.name=Día de las Letras Gallegas
holiday.es-ES48.date=20220517
holiday.es-ES49.name=Día de Canarias
holiday.es-ES49.date=20220530
holiday.es-ES50.name=Día de Castilla-La Mancha
holiday.es-ES50.date=20220531
holiday.es-ES51.name=Día de la Región de Murcia
holiday.es-ES51.date=20220609
holiday.es-ES52.name=Día de La Rioja
holiday.es-ES52.date=20220609
holiday.es-ES53.name=Santiago Apóstol / Día Nacional de Galicia
holiday.es-ES53.date=20220725
holiday.es-ES54.name=Asunción de la Virgen
holiday.es-ES54.date=20220815
holiday.es-ES55.name=Día de Asturias
holiday.es-ES55.date=20220908
holiday.es-ES56.name=Día de Extremadura
holiday.es-ES56.date=20220908
holiday.es-ES57.name=Fiesta Nacional de Cataluña
holiday.es-ES57.date=20220911
holiday.es-ES58.name=Día de la Comunitat Valenciana
holiday.es-ES58.date=20221009
holiday.es-ES59.name=Fiesta Nacional de España
holiday.es-ES59.date=20221012
holiday.es-ES60.name=Todos los Santos
holiday.es-ES60.date=20221101
holiday.es-ES61.name=Día de la Constitución Española
holiday.es-ES61.date=20221206
holiday.es-ES62.name=Inmaculada Concepción
holiday.es-ES62.date=20221208
holiday.es-ES63.name=Natividad del Señor
holiday.es-ES63.date=20221225
