////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// https://www.opensource.org/licenses/mit-license.php
//
////////////////////////////////////////////////////////////////////////////////

#include <algorithm>
#include <cmake.h>
#include <Context.h>
#include <iterator>
#include <unordered_set>
#include <utility>
#include <vector>

////////////////////////////////////////////////////////////////////////////////
// Generates a nag message when there are READY tasks of a higher urgency.
void nag (std::vector <Task>& tasks)
{
  auto msg = Context::getContext ().config.get ("nag");
  if (msg == "")
      return;

  auto pending = Context::getContext ().tdb2.pending.get_tasks ();
  for (auto& t1 : tasks) {
    if (t1.hasTag ("nonag"))
      continue;

    for (auto& t2 : pending) {
      if (t1.get ("uuid") != t2.get ("uuid") &&
        t2.hasTag ("READY")                  &&
        t1.urgency () < t2.urgency ())
      {
        Context::getContext ().footnote (msg);
        return;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
